"""
This module contains default Evaluators.
"""

from .base import Evaluator
from .http import HttpEvaluator
from .postprocess import PostprocessEvaluator
from .preprocess import PreprocessEvaluator
